import bisect

from ..datacube_mappers import DatacubeMapper


class ReducedLatLonMapper(DatacubeMapper):
    def __init__(self, base_axis, mapped_axes, resolution, md5_hash=None, local_area=[], axis_reversed=None):
        # TODO: if local area is not empty list, raise NotImplemented
        self._mapped_axes = mapped_axes
        self._base_axis = base_axis
        self._resolution = resolution
        self._axis_reversed = {mapped_axes[0]: False, mapped_axes[1]: False}
        self._first_axis_vals = self.first_axis_vals()
        self.compressed_grid_axes = [self._mapped_axes[1]]
        if md5_hash is not None:
            self.md5_hash = md5_hash
        else:
            self.md5_hash = _md5_hash.get(resolution, None)
        if self._axis_reversed[mapped_axes[1]]:
            raise NotImplementedError("Reduced lat-lon grid with second axis in decreasing order is not supported")
        if self._axis_reversed[mapped_axes[0]]:
            raise NotImplementedError("Reduced lat-lon grid with first axis in decreasing order is not supported")

    def first_axis_vals(self):
        start_lat = 90
        if self._resolution == 3601:
            start_lat = 89.973092
        resolution = 180 / (self._resolution - 1)
        vals = [round(start_lat - i * resolution, 12) for i in range(self._resolution)]
        return vals

    def map_first_axis(self, lower, upper):
        axis_lines = self._first_axis_vals
        return_vals = [val for val in axis_lines if lower <= val <= upper]
        return return_vals

    def lon_spacing(self):
        if self._resolution == 1441:
            return [
                2,
                6,
                14,
                20,
                26,
                32,
                38,
                44,
                50,
                58,
                64,
                70,
                76,
                82,
                88,
                94,
                102,
                108,
                114,
                120,
                126,
                132,
                138,
                144,
                152,
                158,
                164,
                170,
                176,
                182,
                188,
                196,
                202,
                208,
                214,
                220,
                226,
                232,
                238,
                246,
                252,
                258,
                264,
                270,
                276,
                282,
                290,
                296,
                302,
                308,
                314,
                320,
                326,
                332,
                340,
                346,
                352,
                358,
                364,
                370,
                376,
                382,
                388,
                396,
                402,
                408,
                414,
                420,
                426,
                432,
                438,
                444,
                452,
                458,
                464,
                470,
                476,
                482,
                488,
                494,
                500,
                506,
                512,
                520,
                526,
                532,
                538,
                544,
                550,
                556,
                562,
                568,
                574,
                580,
                586,
                594,
                600,
                606,
                612,
                618,
                624,
                630,
                636,
                642,
                648,
                654,
                660,
                666,
                672,
                678,
                686,
                692,
                698,
                704,
                710,
                716,
                722,
                728,
                734,
                740,
                746,
                752,
                758,
                764,
                770,
                776,
                782,
                788,
                794,
                800,
                806,
                812,
                818,
                824,
                830,
                836,
                842,
                848,
                854,
                860,
                866,
                872,
                878,
                884,
                890,
                896,
                902,
                908,
                914,
                920,
                926,
                932,
                938,
                944,
                950,
                956,
                962,
                968,
                974,
                980,
                986,
                992,
                998,
                1004,
                1010,
                1014,
                1020,
                1026,
                1032,
                1038,
                1044,
                1050,
                1056,
                1062,
                1068,
                1074,
                1080,
                1086,
                1092,
                1096,
                1102,
                1108,
                1114,
                1120,
                1126,
                1132,
                1138,
                1144,
                1148,
                1154,
                1160,
                1166,
                1172,
                1178,
                1184,
                1190,
                1194,
                1200,
                1206,
                1212,
                1218,
                1224,
                1230,
                1234,
                1240,
                1246,
                1252,
                1258,
                1264,
                1268,
                1274,
                1280,
                1286,
                1292,
                1296,
                1302,
                1308,
                1314,
                1320,
                1324,
                1330,
                1336,
                1342,
                1348,
                1352,
                1358,
                1364,
                1370,
                1374,
                1380,
                1386,
                1392,
                1396,
                1402,
                1408,
                1414,
                1418,
                1424,
                1430,
                1436,
                1440,
                1446,
                1452,
                1456,
                1462,
                1468,
                1474,
                1478,
                1484,
                1490,
                1494,
                1500,
                1506,
                1510,
                1516,
                1522,
                1526,
                1532,
                1538,
                1542,
                1548,
                1554,
                1558,
                1564,
                1570,
                1574,
                1580,
                1584,
                1590,
                1596,
                1600,
                1606,
                1610,
                1616,
                1622,
                1626,
                1632,
                1636,
                1642,
                1648,
                1652,
                1658,
                1662,
                1668,
                1672,
                1678,
                1684,
                1688,
                1694,
                1698,
                1704,
                1708,
                1714,
                1718,
                1724,
                1728,
                1734,
                1738,
                1744,
                1748,
                1754,
                1758,
                1764,
                1768,
                1774,
                1778,
                1784,
                1788,
                1794,
                1798,
                1804,
                1808,
                1812,
                1818,
                1822,
                1828,
                1832,
                1838,
                1842,
                1846,
                1852,
                1856,
                1862,
                1866,
                1870,
                1876,
                1880,
                1886,
                1890,
                1894,
                1900,
                1904,
                1908,
                1914,
                1918,
                1922,
                1928,
                1932,
                1936,
                1942,
                1946,
                1950,
                1956,
                1960,
                1964,
                1970,
                1974,
                1978,
                1982,
                1988,
                1992,
                1996,
                2002,
                2006,
                2010,
                2014,
                2020,
                2024,
                2028,
                2032,
                2036,
                2042,
                2046,
                2050,
                2054,
                2060,
                2064,
                2068,
                2072,
                2076,
                2080,
                2086,
                2090,
                2094,
                2098,
                2102,
                2106,
                2112,
                2116,
                2120,
                2124,
                2128,
                2132,
                2136,
                2140,
                2144,
                2150,
                2154,
                2158,
                2162,
                2166,
                2170,
                2174,
                2178,
                2182,
                2186,
                2190,
                2194,
                2198,
                2202,
                2206,
                2210,
                2214,
                2218,
                2222,
                2226,
                2230,
                2234,
                2238,
                2242,
                2246,
                2250,
                2254,
                2258,
                2262,
                2266,
                2270,
                2274,
                2278,
                2282,
                2286,
                2290,
                2292,
                2296,
                2300,
                2304,
                2308,
                2312,
                2316,
                2320,
                2324,
                2326,
                2330,
                2334,
                2338,
                2342,
                2346,
                2348,
                2352,
                2356,
                2360,
                2364,
                2366,
                2370,
                2374,
                2378,
                2382,
                2384,
                2388,
                2392,
                2396,
                2398,
                2402,
                2406,
                2410,
                2412,
                2416,
                2420,
                2422,
                2426,
                2430,
                2432,
                2436,
                2440,
                2442,
                2446,
                2450,
                2452,
                2456,
                2460,
                2462,
                2466,
                2470,
                2472,
                2476,
                2478,
                2482,
                2486,
                2488,
                2492,
                2494,
                2498,
                2500,
                2504,
                2508,
                2510,
                2514,
                2516,
                2520,
                2522,
                2526,
                2528,
                2532,
                2534,
                2538,
                2540,
                2544,
                2546,
                2550,
                2552,
                2556,
                2558,
                2560,
                2564,
                2566,
                2570,
                2572,
                2576,
                2578,
                2580,
                2584,
                2586,
                2590,
                2592,
                2594,
                2598,
                2600,
                2602,
                2606,
                2608,
                2610,
                2614,
                2616,
                2618,
                2622,
                2624,
                2626,
                2628,
                2632,
                2634,
                2636,
                2640,
                2642,
                2644,
                2646,
                2650,
                2652,
                2654,
                2656,
                2658,
                2662,
                2664,
                2666,
                2668,
                2670,
                2674,
                2676,
                2678,
                2680,
                2682,
                2684,
                2686,
                2690,
                2692,
                2694,
                2696,
                2698,
                2700,
                2702,
                2704,
                2706,
                2708,
                2712,
                2714,
                2716,
                2718,
                2720,
                2722,
                2724,
                2726,
                2728,
                2730,
                2732,
                2734,
                2736,
                2738,
                2740,
                2742,
                2744,
                2746,
                2748,
                2750,
                2750,
                2752,
                2754,
                2756,
                2758,
                2760,
                2762,
                2764,
                2766,
                2768,
                2768,
                2770,
                2772,
                2774,
                2776,
                2778,
                2780,
                2780,
                2782,
                2784,
                2786,
                2788,
                2788,
                2790,
                2792,
                2794,
                2794,
                2796,
                2798,
                2800,
                2800,
                2802,
                2804,
                2806,
                2806,
                2808,
                2810,
                2810,
                2812,
                2814,
                2814,
                2816,
                2818,
                2818,
                2820,
                2822,
                2822,
                2824,
                2826,
                2826,
                2828,
                2828,
                2830,
                2832,
                2832,
                2834,
                2834,
                2836,
                2836,
                2838,
                2838,
                2840,
                2842,
                2842,
                2844,
                2844,
                2846,
                2846,
                2846,
                2848,
                2848,
                2850,
                2850,
                2852,
                2852,
                2854,
                2854,
                2856,
                2856,
                2856,
                2858,
                2858,
                2860,
                2860,
                2860,
                2862,
                2862,
                2862,
                2864,
                2864,
                2864,
                2866,
                2866,
                2866,
                2868,
                2868,
                2868,
                2868,
                2870,
                2870,
                2870,
                2872,
                2872,
                2872,
                2872,
                2874,
                2874,
                2874,
                2874,
                2874,
                2876,
                2876,
                2876,
                2876,
                2876,
                2876,
                2878,
                2878,
                2878,
                2878,
                2878,
                2878,
                2878,
                2878,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2880,
                2878,
                2878,
                2878,
                2878,
                2878,
                2878,
                2878,
                2878,
                2876,
                2876,
                2876,
                2876,
                2876,
                2876,
                2874,
                2874,
                2874,
                2874,
                2874,
                2872,
                2872,
                2872,
                2872,
                2870,
                2870,
                2870,
                2868,
                2868,
                2868,
                2868,
                2866,
                2866,
                2866,
                2864,
                2864,
                2864,
                2862,
                2862,
                2862,
                2860,
                2860,
                2860,
                2858,
                2858,
                2856,
                2856,
                2856,
                2854,
                2854,
                2852,
                2852,
                2850,
                2850,
                2848,
                2848,
                2846,
                2846,
                2846,
                2844,
                2844,
                2842,
                2842,
                2840,
                2838,
                2838,
                2836,
                2836,
                2834,
                2834,
                2832,
                2832,
                2830,
                2828,
                2828,
                2826,
                2826,
                2824,
                2822,
                2822,
                2820,
                2818,
                2818,
                2816,
                2814,
                2814,
                2812,
                2810,
                2810,
                2808,
                2806,
                2806,
                2804,
                2802,
                2800,
                2800,
                2798,
                2796,
                2794,
                2794,
                2792,
                2790,
                2788,
                2788,
                2786,
                2784,
                2782,
                2780,
                2780,
                2778,
                2776,
                2774,
                2772,
                2770,
                2768,
                2768,
                2766,
                2764,
                2762,
                2760,
                2758,
                2756,
                2754,
                2752,
                2750,
                2750,
                2748,
                2746,
                2744,
                2742,
                2740,
                2738,
                2736,
                2734,
                2732,
                2730,
                2728,
                2726,
                2724,
                2722,
                2720,
                2718,
                2716,
                2714,
                2712,
                2708,
                2706,
                2704,
                2702,
                2700,
                2698,
                2696,
                2694,
                2692,
                2690,
                2686,
                2684,
                2682,
                2680,
                2678,
                2676,
                2674,
                2670,
                2668,
                2666,
                2664,
                2662,
                2658,
                2656,
                2654,
                2652,
                2650,
                2646,
                2644,
                2642,
                2640,
                2636,
                2634,
                2632,
                2628,
                2626,
                2624,
                2622,
                2618,
                2616,
                2614,
                2610,
                2608,
                2606,
                2602,
                2600,
                2598,
                2594,
                2592,
                2590,
                2586,
                2584,
                2580,
                2578,
                2576,
                2572,
                2570,
                2566,
                2564,
                2560,
                2558,
                2556,
                2552,
                2550,
                2546,
                2544,
                2540,
                2538,
                2534,
                2532,
                2528,
                2526,
                2522,
                2520,
                2516,
                2514,
                2510,
                2508,
                2504,
                2500,
                2498,
                2494,
                2492,
                2488,
                2486,
                2482,
                2478,
                2476,
                2472,
                2470,
                2466,
                2462,
                2460,
                2456,
                2452,
                2450,
                2446,
                2442,
                2440,
                2436,
                2432,
                2430,
                2426,
                2422,
                2420,
                2416,
                2412,
                2410,
                2406,
                2402,
                2398,
                2396,
                2392,
                2388,
                2384,
                2382,
                2378,
                2374,
                2370,
                2366,
                2364,
                2360,
                2356,
                2352,
                2348,
                2346,
                2342,
                2338,
                2334,
                2330,
                2326,
                2324,
                2320,
                2316,
                2312,
                2308,
                2304,
                2300,
                2296,
                2292,
                2290,
                2286,
                2282,
                2278,
                2274,
                2270,
                2266,
                2262,
                2258,
                2254,
                2250,
                2246,
                2242,
                2238,
                2234,
                2230,
                2226,
                2222,
                2218,
                2214,
                2210,
                2206,
                2202,
                2198,
                2194,
                2190,
                2186,
                2182,
                2178,
                2174,
                2170,
                2166,
                2162,
                2158,
                2154,
                2150,
                2144,
                2140,
                2136,
                2132,
                2128,
                2124,
                2120,
                2116,
                2112,
                2106,
                2102,
                2098,
                2094,
                2090,
                2086,
                2080,
                2076,
                2072,
                2068,
                2064,
                2060,
                2054,
                2050,
                2046,
                2042,
                2036,
                2032,
                2028,
                2024,
                2020,
                2014,
                2010,
                2006,
                2002,
                1996,
                1992,
                1988,
                1982,
                1978,
                1974,
                1970,
                1964,
                1960,
                1956,
                1950,
                1946,
                1942,
                1936,
                1932,
                1928,
                1922,
                1918,
                1914,
                1908,
                1904,
                1900,
                1894,
                1890,
                1886,
                1880,
                1876,
                1870,
                1866,
                1862,
                1856,
                1852,
                1846,
                1842,
                1838,
                1832,
                1828,
                1822,
                1818,
                1812,
                1808,
                1804,
                1798,
                1794,
                1788,
                1784,
                1778,
                1774,
                1768,
                1764,
                1758,
                1754,
                1748,
                1744,
                1738,
                1734,
                1728,
                1724,
                1718,
                1714,
                1708,
                1704,
                1698,
                1694,
                1688,
                1684,
                1678,
                1672,
                1668,
                1662,
                1658,
                1652,
                1648,
                1642,
                1636,
                1632,
                1626,
                1622,
                1616,
                1610,
                1606,
                1600,
                1596,
                1590,
                1584,
                1580,
                1574,
                1570,
                1564,
                1558,
                1554,
                1548,
                1542,
                1538,
                1532,
                1526,
                1522,
                1516,
                1510,
                1506,
                1500,
                1494,
                1490,
                1484,
                1478,
                1474,
                1468,
                1462,
                1456,
                1452,
                1446,
                1440,
                1436,
                1430,
                1424,
                1418,
                1414,
                1408,
                1402,
                1396,
                1392,
                1386,
                1380,
                1374,
                1370,
                1364,
                1358,
                1352,
                1348,
                1342,
                1336,
                1330,
                1324,
                1320,
                1314,
                1308,
                1302,
                1296,
                1292,
                1286,
                1280,
                1274,
                1268,
                1264,
                1258,
                1252,
                1246,
                1240,
                1234,
                1230,
                1224,
                1218,
                1212,
                1206,
                1200,
                1194,
                1190,
                1184,
                1178,
                1172,
                1166,
                1160,
                1154,
                1148,
                1144,
                1138,
                1132,
                1126,
                1120,
                1114,
                1108,
                1102,
                1096,
                1092,
                1086,
                1080,
                1074,
                1068,
                1062,
                1056,
                1050,
                1044,
                1038,
                1032,
                1026,
                1020,
                1014,
                1010,
                1004,
                998,
                992,
                986,
                980,
                974,
                968,
                962,
                956,
                950,
                944,
                938,
                932,
                926,
                920,
                914,
                908,
                902,
                896,
                890,
                884,
                878,
                872,
                866,
                860,
                854,
                848,
                842,
                836,
                830,
                824,
                818,
                812,
                806,
                800,
                794,
                788,
                782,
                776,
                770,
                764,
                758,
                752,
                746,
                740,
                734,
                728,
                722,
                716,
                710,
                704,
                698,
                692,
                686,
                678,
                672,
                666,
                660,
                654,
                648,
                642,
                636,
                630,
                624,
                618,
                612,
                606,
                600,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
            ]
        if self._resolution == 3601:
            return [
                2,
                6,
                14,
                20,
                26,
                32,
                38,
                44,
                50,
                58,
                64,
                70,
                76,
                82,
                88,
                94,
                102,
                108,
                114,
                120,
                126,
                132,
                138,
                146,
                152,
                158,
                164,
                170,
                176,
                182,
                188,
                196,
                202,
                208,
                214,
                220,
                226,
                232,
                240,
                246,
                252,
                258,
                264,
                270,
                276,
                284,
                290,
                296,
                302,
                308,
                314,
                320,
                328,
                334,
                340,
                346,
                352,
                358,
                364,
                372,
                378,
                384,
                390,
                396,
                402,
                408,
                414,
                422,
                428,
                434,
                440,
                446,
                452,
                458,
                466,
                472,
                478,
                484,
                490,
                496,
                502,
                510,
                516,
                522,
                528,
                534,
                540,
                546,
                552,
                560,
                566,
                572,
                578,
                584,
                590,
                596,
                602,
                610,
                616,
                622,
                628,
                634,
                640,
                646,
                654,
                660,
                666,
                672,
                678,
                684,
                690,
                696,
                704,
                710,
                716,
                722,
                728,
                734,
                740,
                746,
                754,
                760,
                766,
                772,
                778,
                784,
                790,
                796,
                804,
                810,
                816,
                822,
                828,
                834,
                840,
                846,
                854,
                860,
                866,
                872,
                878,
                884,
                890,
                896,
                902,
                910,
                916,
                922,
                928,
                934,
                940,
                946,
                952,
                958,
                966,
                972,
                978,
                984,
                990,
                996,
                1002,
                1008,
                1014,
                1022,
                1028,
                1034,
                1040,
                1046,
                1052,
                1058,
                1064,
                1070,
                1078,
                1084,
                1090,
                1096,
                1102,
                1108,
                1114,
                1120,
                1126,
                1134,
                1140,
                1146,
                1152,
                1158,
                1164,
                1170,
                1176,
                1182,
                1188,
                1196,
                1202,
                1208,
                1214,
                1220,
                1226,
                1232,
                1238,
                1244,
                1250,
                1256,
                1264,
                1270,
                1276,
                1282,
                1288,
                1294,
                1300,
                1306,
                1312,
                1318,
                1324,
                1332,
                1338,
                1344,
                1350,
                1356,
                1362,
                1368,
                1374,
                1380,
                1386,
                1392,
                1398,
                1406,
                1412,
                1418,
                1424,
                1430,
                1436,
                1442,
                1448,
                1454,
                1460,
                1466,
                1472,
                1480,
                1486,
                1492,
                1498,
                1504,
                1510,
                1516,
                1522,
                1528,
                1534,
                1540,
                1546,
                1552,
                1558,
                1564,
                1572,
                1578,
                1584,
                1590,
                1596,
                1602,
                1608,
                1614,
                1620,
                1626,
                1632,
                1638,
                1644,
                1650,
                1656,
                1662,
                1670,
                1676,
                1682,
                1688,
                1694,
                1700,
                1706,
                1712,
                1718,
                1724,
                1730,
                1736,
                1742,
                1748,
                1754,
                1760,
                1766,
                1772,
                1778,
                1784,
                1792,
                1798,
                1804,
                1810,
                1816,
                1822,
                1828,
                1834,
                1840,
                1846,
                1852,
                1858,
                1864,
                1870,
                1876,
                1882,
                1888,
                1894,
                1900,
                1906,
                1912,
                1918,
                1924,
                1930,
                1936,
                1942,
                1948,
                1954,
                1960,
                1966,
                1974,
                1980,
                1986,
                1992,
                1998,
                2004,
                2010,
                2016,
                2022,
                2028,
                2034,
                2040,
                2046,
                2052,
                2058,
                2064,
                2070,
                2076,
                2082,
                2088,
                2094,
                2100,
                2106,
                2112,
                2118,
                2124,
                2130,
                2136,
                2142,
                2148,
                2154,
                2160,
                2166,
                2172,
                2178,
                2184,
                2190,
                2196,
                2202,
                2208,
                2214,
                2220,
                2226,
                2232,
                2238,
                2244,
                2250,
                2256,
                2262,
                2268,
                2274,
                2280,
                2286,
                2292,
                2298,
                2302,
                2308,
                2314,
                2320,
                2326,
                2332,
                2338,
                2344,
                2350,
                2356,
                2362,
                2368,
                2374,
                2380,
                2386,
                2392,
                2398,
                2404,
                2410,
                2416,
                2422,
                2428,
                2434,
                2440,
                2446,
                2452,
                2458,
                2464,
                2468,
                2474,
                2480,
                2486,
                2492,
                2498,
                2504,
                2510,
                2516,
                2522,
                2528,
                2534,
                2540,
                2546,
                2552,
                2558,
                2564,
                2570,
                2574,
                2580,
                2586,
                2592,
                2598,
                2604,
                2610,
                2616,
                2622,
                2628,
                2634,
                2640,
                2646,
                2650,
                2656,
                2662,
                2668,
                2674,
                2680,
                2686,
                2692,
                2698,
                2704,
                2710,
                2716,
                2720,
                2726,
                2732,
                2738,
                2744,
                2750,
                2756,
                2762,
                2768,
                2774,
                2780,
                2784,
                2790,
                2796,
                2802,
                2808,
                2814,
                2820,
                2826,
                2832,
                2836,
                2842,
                2848,
                2854,
                2860,
                2866,
                2872,
                2878,
                2884,
                2888,
                2894,
                2900,
                2906,
                2912,
                2918,
                2924,
                2930,
                2934,
                2940,
                2946,
                2952,
                2958,
                2964,
                2970,
                2974,
                2980,
                2986,
                2992,
                2998,
                3004,
                3010,
                3014,
                3020,
                3026,
                3032,
                3038,
                3044,
                3050,
                3054,
                3060,
                3066,
                3072,
                3078,
                3084,
                3088,
                3094,
                3100,
                3106,
                3112,
                3118,
                3122,
                3128,
                3134,
                3140,
                3146,
                3152,
                3156,
                3162,
                3168,
                3174,
                3180,
                3184,
                3190,
                3196,
                3202,
                3208,
                3214,
                3218,
                3224,
                3230,
                3236,
                3242,
                3246,
                3252,
                3258,
                3264,
                3270,
                3274,
                3280,
                3286,
                3292,
                3298,
                3302,
                3308,
                3314,
                3320,
                3326,
                3330,
                3336,
                3342,
                3348,
                3352,
                3358,
                3364,
                3370,
                3376,
                3380,
                3386,
                3392,
                3398,
                3402,
                3408,
                3414,
                3420,
                3424,
                3430,
                3436,
                3442,
                3448,
                3452,
                3458,
                3464,
                3470,
                3474,
                3480,
                3486,
                3492,
                3496,
                3502,
                3508,
                3514,
                3518,
                3524,
                3530,
                3536,
                3540,
                3546,
                3552,
                3556,
                3562,
                3568,
                3574,
                3578,
                3584,
                3590,
                3596,
                3600,
                3606,
                3612,
                3616,
                3622,
                3628,
                3634,
                3638,
                3644,
                3650,
                3654,
                3660,
                3666,
                3672,
                3676,
                3682,
                3688,
                3692,
                3698,
                3704,
                3708,
                3714,
                3720,
                3724,
                3730,
                3736,
                3742,
                3746,
                3752,
                3758,
                3762,
                3768,
                3774,
                3778,
                3784,
                3790,
                3794,
                3800,
                3806,
                3810,
                3816,
                3822,
                3826,
                3832,
                3838,
                3842,
                3848,
                3854,
                3858,
                3864,
                3870,
                3874,
                3880,
                3884,
                3890,
                3896,
                3900,
                3906,
                3912,
                3916,
                3922,
                3928,
                3932,
                3938,
                3942,
                3948,
                3954,
                3958,
                3964,
                3970,
                3974,
                3980,
                3984,
                3990,
                3996,
                4000,
                4006,
                4012,
                4016,
                4022,
                4026,
                4032,
                4038,
                4042,
                4048,
                4052,
                4058,
                4064,
                4068,
                4074,
                4078,
                4084,
                4088,
                4094,
                4100,
                4104,
                4110,
                4114,
                4120,
                4126,
                4130,
                4136,
                4140,
                4146,
                4150,
                4156,
                4162,
                4166,
                4172,
                4176,
                4182,
                4186,
                4192,
                4196,
                4202,
                4208,
                4212,
                4218,
                4222,
                4228,
                4232,
                4238,
                4242,
                4248,
                4252,
                4258,
                4262,
                4268,
                4274,
                4278,
                4284,
                4288,
                4294,
                4298,
                4304,
                4308,
                4314,
                4318,
                4324,
                4328,
                4334,
                4338,
                4344,
                4348,
                4354,
                4358,
                4364,
                4368,
                4374,
                4378,
                4384,
                4388,
                4394,
                4398,
                4404,
                4408,
                4414,
                4418,
                4424,
                4428,
                4434,
                4438,
                4444,
                4448,
                4454,
                4458,
                4462,
                4468,
                4472,
                4478,
                4482,
                4488,
                4492,
                4498,
                4502,
                4508,
                4512,
                4516,
                4522,
                4526,
                4532,
                4536,
                4542,
                4546,
                4552,
                4556,
                4560,
                4566,
                4570,
                4576,
                4580,
                4586,
                4590,
                4594,
                4600,
                4604,
                4610,
                4614,
                4618,
                4624,
                4628,
                4634,
                4638,
                4642,
                4648,
                4652,
                4658,
                4662,
                4666,
                4672,
                4676,
                4682,
                4686,
                4690,
                4696,
                4700,
                4706,
                4710,
                4714,
                4720,
                4724,
                4728,
                4734,
                4738,
                4744,
                4748,
                4752,
                4758,
                4762,
                4766,
                4772,
                4776,
                4780,
                4786,
                4790,
                4794,
                4800,
                4804,
                4808,
                4814,
                4818,
                4822,
                4828,
                4832,
                4836,
                4842,
                4846,
                4850,
                4856,
                4860,
                4864,
                4870,
                4874,
                4878,
                4884,
                4888,
                4892,
                4898,
                4902,
                4906,
                4910,
                4916,
                4920,
                4924,
                4930,
                4934,
                4938,
                4942,
                4948,
                4952,
                4956,
                4962,
                4966,
                4970,
                4974,
                4980,
                4984,
                4988,
                4992,
                4998,
                5002,
                5006,
                5012,
                5016,
                5020,
                5024,
                5030,
                5034,
                5038,
                5042,
                5048,
                5052,
                5056,
                5060,
                5064,
                5070,
                5074,
                5078,
                5082,
                5088,
                5092,
                5096,
                5100,
                5104,
                5110,
                5114,
                5118,
                5122,
                5128,
                5132,
                5136,
                5140,
                5144,
                5150,
                5154,
                5158,
                5162,
                5166,
                5172,
                5176,
                5180,
                5184,
                5188,
                5192,
                5198,
                5202,
                5206,
                5210,
                5214,
                5218,
                5224,
                5228,
                5232,
                5236,
                5240,
                5244,
                5250,
                5254,
                5258,
                5262,
                5266,
                5270,
                5274,
                5280,
                5284,
                5288,
                5292,
                5296,
                5300,
                5304,
                5308,
                5314,
                5318,
                5322,
                5326,
                5330,
                5334,
                5338,
                5342,
                5346,
                5352,
                5356,
                5360,
                5364,
                5368,
                5372,
                5376,
                5380,
                5384,
                5388,
                5392,
                5398,
                5402,
                5406,
                5410,
                5414,
                5418,
                5422,
                5426,
                5430,
                5434,
                5438,
                5442,
                5446,
                5450,
                5454,
                5460,
                5464,
                5468,
                5472,
                5476,
                5480,
                5484,
                5488,
                5492,
                5496,
                5500,
                5504,
                5508,
                5512,
                5516,
                5520,
                5524,
                5528,
                5532,
                5536,
                5540,
                5544,
                5548,
                5552,
                5556,
                5560,
                5564,
                5568,
                5572,
                5576,
                5580,
                5584,
                5588,
                5592,
                5596,
                5600,
                5604,
                5608,
                5612,
                5616,
                5620,
                5624,
                5628,
                5632,
                5636,
                5640,
                5644,
                5646,
                5650,
                5654,
                5658,
                5662,
                5666,
                5670,
                5674,
                5678,
                5682,
                5686,
                5690,
                5694,
                5698,
                5702,
                5704,
                5708,
                5712,
                5716,
                5720,
                5724,
                5728,
                5732,
                5736,
                5740,
                5744,
                5746,
                5750,
                5754,
                5758,
                5762,
                5766,
                5770,
                5774,
                5778,
                5780,
                5784,
                5788,
                5792,
                5796,
                5800,
                5804,
                5806,
                5810,
                5814,
                5818,
                5822,
                5826,
                5830,
                5832,
                5836,
                5840,
                5844,
                5848,
                5852,
                5854,
                5858,
                5862,
                5866,
                5870,
                5874,
                5876,
                5880,
                5884,
                5888,
                5892,
                5894,
                5898,
                5902,
                5906,
                5910,
                5912,
                5916,
                5920,
                5924,
                5928,
                5930,
                5934,
                5938,
                5942,
                5944,
                5948,
                5952,
                5956,
                5960,
                5962,
                5966,
                5970,
                5974,
                5976,
                5980,
                5984,
                5988,
                5990,
                5994,
                5998,
                6002,
                6004,
                6008,
                6012,
                6014,
                6018,
                6022,
                6026,
                6028,
                6032,
                6036,
                6038,
                6042,
                6046,
                6050,
                6052,
                6056,
                6060,
                6062,
                6066,
                6070,
                6072,
                6076,
                6080,
                6084,
                6086,
                6090,
                6094,
                6096,
                6100,
                6104,
                6106,
                6110,
                6114,
                6116,
                6120,
                6124,
                6126,
                6130,
                6132,
                6136,
                6140,
                6142,
                6146,
                6150,
                6152,
                6156,
                6160,
                6162,
                6166,
                6168,
                6172,
                6176,
                6178,
                6182,
                6186,
                6188,
                6192,
                6194,
                6198,
                6202,
                6204,
                6208,
                6210,
                6214,
                6216,
                6220,
                6224,
                6226,
                6230,
                6232,
                6236,
                6240,
                6242,
                6246,
                6248,
                6252,
                6254,
                6258,
                6260,
                6264,
                6268,
                6270,
                6274,
                6276,
                6280,
                6282,
                6286,
                6288,
                6292,
                6294,
                6298,
                6300,
                6304,
                6306,
                6310,
                6312,
                6316,
                6318,
                6322,
                6324,
                6328,
                6330,
                6334,
                6336,
                6340,
                6342,
                6346,
                6348,
                6352,
                6354,
                6358,
                6360,
                6364,
                6366,
                6370,
                6372,
                6376,
                6378,
                6382,
                6384,
                6386,
                6390,
                6392,
                6396,
                6398,
                6402,
                6404,
                6408,
                6410,
                6412,
                6416,
                6418,
                6422,
                6424,
                6428,
                6430,
                6432,
                6436,
                6438,
                6442,
                6444,
                6446,
                6450,
                6452,
                6456,
                6458,
                6460,
                6464,
                6466,
                6470,
                6472,
                6474,
                6478,
                6480,
                6482,
                6486,
                6488,
                6490,
                6494,
                6496,
                6500,
                6502,
                6504,
                6508,
                6510,
                6512,
                6516,
                6518,
                6520,
                6524,
                6526,
                6528,
                6532,
                6534,
                6536,
                6540,
                6542,
                6544,
                6546,
                6550,
                6552,
                6554,
                6558,
                6560,
                6562,
                6566,
                6568,
                6570,
                6572,
                6576,
                6578,
                6580,
                6584,
                6586,
                6588,
                6590,
                6594,
                6596,
                6598,
                6600,
                6604,
                6606,
                6608,
                6610,
                6614,
                6616,
                6618,
                6620,
                6624,
                6626,
                6628,
                6630,
                6634,
                6636,
                6638,
                6640,
                6642,
                6646,
                6648,
                6650,
                6652,
                6654,
                6658,
                6660,
                6662,
                6664,
                6666,
                6670,
                6672,
                6674,
                6676,
                6678,
                6680,
                6684,
                6686,
                6688,
                6690,
                6692,
                6694,
                6698,
                6700,
                6702,
                6704,
                6706,
                6708,
                6710,
                6714,
                6716,
                6718,
                6720,
                6722,
                6724,
                6726,
                6730,
                6732,
                6734,
                6736,
                6738,
                6740,
                6742,
                6744,
                6746,
                6748,
                6752,
                6754,
                6756,
                6758,
                6760,
                6762,
                6764,
                6766,
                6768,
                6770,
                6772,
                6774,
                6776,
                6780,
                6782,
                6784,
                6786,
                6788,
                6790,
                6792,
                6794,
                6796,
                6798,
                6800,
                6802,
                6804,
                6806,
                6808,
                6810,
                6812,
                6814,
                6816,
                6818,
                6820,
                6822,
                6824,
                6826,
                6828,
                6830,
                6832,
                6834,
                6836,
                6838,
                6840,
                6842,
                6844,
                6846,
                6848,
                6850,
                6852,
                6854,
                6856,
                6858,
                6860,
                6862,
                6864,
                6866,
                6868,
                6870,
                6872,
                6872,
                6874,
                6876,
                6878,
                6880,
                6882,
                6884,
                6886,
                6888,
                6890,
                6892,
                6894,
                6896,
                6896,
                6898,
                6900,
                6902,
                6904,
                6906,
                6908,
                6910,
                6912,
                6912,
                6914,
                6916,
                6918,
                6920,
                6922,
                6924,
                6926,
                6926,
                6928,
                6930,
                6932,
                6934,
                6936,
                6936,
                6938,
                6940,
                6942,
                6944,
                6946,
                6946,
                6948,
                6950,
                6952,
                6954,
                6956,
                6956,
                6958,
                6960,
                6962,
                6964,
                6964,
                6966,
                6968,
                6970,
                6972,
                6972,
                6974,
                6976,
                6978,
                6978,
                6980,
                6982,
                6984,
                6986,
                6986,
                6988,
                6990,
                6992,
                6992,
                6994,
                6996,
                6998,
                6998,
                7000,
                7002,
                7004,
                7004,
                7006,
                7008,
                7008,
                7010,
                7012,
                7014,
                7014,
                7016,
                7018,
                7018,
                7020,
                7022,
                7022,
                7024,
                7026,
                7028,
                7028,
                7030,
                7032,
                7032,
                7034,
                7036,
                7036,
                7038,
                7040,
                7040,
                7042,
                7044,
                7044,
                7046,
                7048,
                7048,
                7050,
                7050,
                7052,
                7054,
                7054,
                7056,
                7058,
                7058,
                7060,
                7060,
                7062,
                7064,
                7064,
                7066,
                7068,
                7068,
                7070,
                7070,
                7072,
                7072,
                7074,
                7076,
                7076,
                7078,
                7078,
                7080,
                7082,
                7082,
                7084,
                7084,
                7086,
                7086,
                7088,
                7088,
                7090,
                7092,
                7092,
                7094,
                7094,
                7096,
                7096,
                7098,
                7098,
                7100,
                7100,
                7102,
                7102,
                7104,
                7104,
                7106,
                7106,
                7108,
                7108,
                7110,
                7110,
                7112,
                7112,
                7114,
                7114,
                7116,
                7116,
                7118,
                7118,
                7120,
                7120,
                7122,
                7122,
                7124,
                7124,
                7126,
                7126,
                7126,
                7128,
                7128,
                7130,
                7130,
                7132,
                7132,
                7134,
                7134,
                7134,
                7136,
                7136,
                7138,
                7138,
                7138,
                7140,
                7140,
                7142,
                7142,
                7142,
                7144,
                7144,
                7146,
                7146,
                7146,
                7148,
                7148,
                7150,
                7150,
                7150,
                7152,
                7152,
                7152,
                7154,
                7154,
                7154,
                7156,
                7156,
                7158,
                7158,
                7158,
                7160,
                7160,
                7160,
                7162,
                7162,
                7162,
                7164,
                7164,
                7164,
                7164,
                7166,
                7166,
                7166,
                7168,
                7168,
                7168,
                7170,
                7170,
                7170,
                7170,
                7172,
                7172,
                7172,
                7174,
                7174,
                7174,
                7174,
                7176,
                7176,
                7176,
                7176,
                7178,
                7178,
                7178,
                7178,
                7180,
                7180,
                7180,
                7180,
                7182,
                7182,
                7182,
                7182,
                7182,
                7184,
                7184,
                7184,
                7184,
                7186,
                7186,
                7186,
                7186,
                7186,
                7188,
                7188,
                7188,
                7188,
                7188,
                7188,
                7190,
                7190,
                7190,
                7190,
                7190,
                7190,
                7192,
                7192,
                7192,
                7192,
                7192,
                7192,
                7194,
                7194,
                7194,
                7194,
                7194,
                7194,
                7194,
                7194,
                7196,
                7196,
                7196,
                7196,
                7196,
                7196,
                7196,
                7196,
                7196,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7200,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7198,
                7196,
                7196,
                7196,
                7196,
                7196,
                7196,
                7196,
                7196,
                7196,
                7194,
                7194,
                7194,
                7194,
                7194,
                7194,
                7194,
                7194,
                7192,
                7192,
                7192,
                7192,
                7192,
                7192,
                7190,
                7190,
                7190,
                7190,
                7190,
                7190,
                7188,
                7188,
                7188,
                7188,
                7188,
                7188,
                7186,
                7186,
                7186,
                7186,
                7186,
                7184,
                7184,
                7184,
                7184,
                7182,
                7182,
                7182,
                7182,
                7182,
                7180,
                7180,
                7180,
                7180,
                7178,
                7178,
                7178,
                7178,
                7176,
                7176,
                7176,
                7176,
                7174,
                7174,
                7174,
                7174,
                7172,
                7172,
                7172,
                7170,
                7170,
                7170,
                7170,
                7168,
                7168,
                7168,
                7166,
                7166,
                7166,
                7164,
                7164,
                7164,
                7164,
                7162,
                7162,
                7162,
                7160,
                7160,
                7160,
                7158,
                7158,
                7158,
                7156,
                7156,
                7154,
                7154,
                7154,
                7152,
                7152,
                7152,
                7150,
                7150,
                7150,
                7148,
                7148,
                7146,
                7146,
                7146,
                7144,
                7144,
                7142,
                7142,
                7142,
                7140,
                7140,
                7138,
                7138,
                7138,
                7136,
                7136,
                7134,
                7134,
                7134,
                7132,
                7132,
                7130,
                7130,
                7128,
                7128,
                7126,
                7126,
                7126,
                7124,
                7124,
                7122,
                7122,
                7120,
                7120,
                7118,
                7118,
                7116,
                7116,
                7114,
                7114,
                7112,
                7112,
                7110,
                7110,
                7108,
                7108,
                7106,
                7106,
                7104,
                7104,
                7102,
                7102,
                7100,
                7100,
                7098,
                7098,
                7096,
                7096,
                7094,
                7094,
                7092,
                7092,
                7090,
                7088,
                7088,
                7086,
                7086,
                7084,
                7084,
                7082,
                7082,
                7080,
                7078,
                7078,
                7076,
                7076,
                7074,
                7072,
                7072,
                7070,
                7070,
                7068,
                7068,
                7066,
                7064,
                7064,
                7062,
                7060,
                7060,
                7058,
                7058,
                7056,
                7054,
                7054,
                7052,
                7050,
                7050,
                7048,
                7048,
                7046,
                7044,
                7044,
                7042,
                7040,
                7040,
                7038,
                7036,
                7036,
                7034,
                7032,
                7032,
                7030,
                7028,
                7028,
                7026,
                7024,
                7022,
                7022,
                7020,
                7018,
                7018,
                7016,
                7014,
                7014,
                7012,
                7010,
                7008,
                7008,
                7006,
                7004,
                7004,
                7002,
                7000,
                6998,
                6998,
                6996,
                6994,
                6992,
                6992,
                6990,
                6988,
                6986,
                6986,
                6984,
                6982,
                6980,
                6978,
                6978,
                6976,
                6974,
                6972,
                6972,
                6970,
                6968,
                6966,
                6964,
                6964,
                6962,
                6960,
                6958,
                6956,
                6956,
                6954,
                6952,
                6950,
                6948,
                6946,
                6946,
                6944,
                6942,
                6940,
                6938,
                6936,
                6936,
                6934,
                6932,
                6930,
                6928,
                6926,
                6926,
                6924,
                6922,
                6920,
                6918,
                6916,
                6914,
                6912,
                6912,
                6910,
                6908,
                6906,
                6904,
                6902,
                6900,
                6898,
                6896,
                6896,
                6894,
                6892,
                6890,
                6888,
                6886,
                6884,
                6882,
                6880,
                6878,
                6876,
                6874,
                6872,
                6872,
                6870,
                6868,
                6866,
                6864,
                6862,
                6860,
                6858,
                6856,
                6854,
                6852,
                6850,
                6848,
                6846,
                6844,
                6842,
                6840,
                6838,
                6836,
                6834,
                6832,
                6830,
                6828,
                6826,
                6824,
                6822,
                6820,
                6818,
                6816,
                6814,
                6812,
                6810,
                6808,
                6806,
                6804,
                6802,
                6800,
                6798,
                6796,
                6794,
                6792,
                6790,
                6788,
                6786,
                6784,
                6782,
                6780,
                6776,
                6774,
                6772,
                6770,
                6768,
                6766,
                6764,
                6762,
                6760,
                6758,
                6756,
                6754,
                6752,
                6748,
                6746,
                6744,
                6742,
                6740,
                6738,
                6736,
                6734,
                6732,
                6730,
                6726,
                6724,
                6722,
                6720,
                6718,
                6716,
                6714,
                6710,
                6708,
                6706,
                6704,
                6702,
                6700,
                6698,
                6694,
                6692,
                6690,
                6688,
                6686,
                6684,
                6680,
                6678,
                6676,
                6674,
                6672,
                6670,
                6666,
                6664,
                6662,
                6660,
                6658,
                6654,
                6652,
                6650,
                6648,
                6646,
                6642,
                6640,
                6638,
                6636,
                6634,
                6630,
                6628,
                6626,
                6624,
                6620,
                6618,
                6616,
                6614,
                6610,
                6608,
                6606,
                6604,
                6600,
                6598,
                6596,
                6594,
                6590,
                6588,
                6586,
                6584,
                6580,
                6578,
                6576,
                6572,
                6570,
                6568,
                6566,
                6562,
                6560,
                6558,
                6554,
                6552,
                6550,
                6546,
                6544,
                6542,
                6540,
                6536,
                6534,
                6532,
                6528,
                6526,
                6524,
                6520,
                6518,
                6516,
                6512,
                6510,
                6508,
                6504,
                6502,
                6500,
                6496,
                6494,
                6490,
                6488,
                6486,
                6482,
                6480,
                6478,
                6474,
                6472,
                6470,
                6466,
                6464,
                6460,
                6458,
                6456,
                6452,
                6450,
                6446,
                6444,
                6442,
                6438,
                6436,
                6432,
                6430,
                6428,
                6424,
                6422,
                6418,
                6416,
                6412,
                6410,
                6408,
                6404,
                6402,
                6398,
                6396,
                6392,
                6390,
                6386,
                6384,
                6382,
                6378,
                6376,
                6372,
                6370,
                6366,
                6364,
                6360,
                6358,
                6354,
                6352,
                6348,
                6346,
                6342,
                6340,
                6336,
                6334,
                6330,
                6328,
                6324,
                6322,
                6318,
                6316,
                6312,
                6310,
                6306,
                6304,
                6300,
                6298,
                6294,
                6292,
                6288,
                6286,
                6282,
                6280,
                6276,
                6274,
                6270,
                6268,
                6264,
                6260,
                6258,
                6254,
                6252,
                6248,
                6246,
                6242,
                6240,
                6236,
                6232,
                6230,
                6226,
                6224,
                6220,
                6216,
                6214,
                6210,
                6208,
                6204,
                6202,
                6198,
                6194,
                6192,
                6188,
                6186,
                6182,
                6178,
                6176,
                6172,
                6168,
                6166,
                6162,
                6160,
                6156,
                6152,
                6150,
                6146,
                6142,
                6140,
                6136,
                6132,
                6130,
                6126,
                6124,
                6120,
                6116,
                6114,
                6110,
                6106,
                6104,
                6100,
                6096,
                6094,
                6090,
                6086,
                6084,
                6080,
                6076,
                6072,
                6070,
                6066,
                6062,
                6060,
                6056,
                6052,
                6050,
                6046,
                6042,
                6038,
                6036,
                6032,
                6028,
                6026,
                6022,
                6018,
                6014,
                6012,
                6008,
                6004,
                6002,
                5998,
                5994,
                5990,
                5988,
                5984,
                5980,
                5976,
                5974,
                5970,
                5966,
                5962,
                5960,
                5956,
                5952,
                5948,
                5944,
                5942,
                5938,
                5934,
                5930,
                5928,
                5924,
                5920,
                5916,
                5912,
                5910,
                5906,
                5902,
                5898,
                5894,
                5892,
                5888,
                5884,
                5880,
                5876,
                5874,
                5870,
                5866,
                5862,
                5858,
                5854,
                5852,
                5848,
                5844,
                5840,
                5836,
                5832,
                5830,
                5826,
                5822,
                5818,
                5814,
                5810,
                5806,
                5804,
                5800,
                5796,
                5792,
                5788,
                5784,
                5780,
                5778,
                5774,
                5770,
                5766,
                5762,
                5758,
                5754,
                5750,
                5746,
                5744,
                5740,
                5736,
                5732,
                5728,
                5724,
                5720,
                5716,
                5712,
                5708,
                5704,
                5702,
                5698,
                5694,
                5690,
                5686,
                5682,
                5678,
                5674,
                5670,
                5666,
                5662,
                5658,
                5654,
                5650,
                5646,
                5644,
                5640,
                5636,
                5632,
                5628,
                5624,
                5620,
                5616,
                5612,
                5608,
                5604,
                5600,
                5596,
                5592,
                5588,
                5584,
                5580,
                5576,
                5572,
                5568,
                5564,
                5560,
                5556,
                5552,
                5548,
                5544,
                5540,
                5536,
                5532,
                5528,
                5524,
                5520,
                5516,
                5512,
                5508,
                5504,
                5500,
                5496,
                5492,
                5488,
                5484,
                5480,
                5476,
                5472,
                5468,
                5464,
                5460,
                5454,
                5450,
                5446,
                5442,
                5438,
                5434,
                5430,
                5426,
                5422,
                5418,
                5414,
                5410,
                5406,
                5402,
                5398,
                5392,
                5388,
                5384,
                5380,
                5376,
                5372,
                5368,
                5364,
                5360,
                5356,
                5352,
                5346,
                5342,
                5338,
                5334,
                5330,
                5326,
                5322,
                5318,
                5314,
                5308,
                5304,
                5300,
                5296,
                5292,
                5288,
                5284,
                5280,
                5274,
                5270,
                5266,
                5262,
                5258,
                5254,
                5250,
                5244,
                5240,
                5236,
                5232,
                5228,
                5224,
                5218,
                5214,
                5210,
                5206,
                5202,
                5198,
                5192,
                5188,
                5184,
                5180,
                5176,
                5172,
                5166,
                5162,
                5158,
                5154,
                5150,
                5144,
                5140,
                5136,
                5132,
                5128,
                5122,
                5118,
                5114,
                5110,
                5104,
                5100,
                5096,
                5092,
                5088,
                5082,
                5078,
                5074,
                5070,
                5064,
                5060,
                5056,
                5052,
                5048,
                5042,
                5038,
                5034,
                5030,
                5024,
                5020,
                5016,
                5012,
                5006,
                5002,
                4998,
                4992,
                4988,
                4984,
                4980,
                4974,
                4970,
                4966,
                4962,
                4956,
                4952,
                4948,
                4942,
                4938,
                4934,
                4930,
                4924,
                4920,
                4916,
                4910,
                4906,
                4902,
                4898,
                4892,
                4888,
                4884,
                4878,
                4874,
                4870,
                4864,
                4860,
                4856,
                4850,
                4846,
                4842,
                4836,
                4832,
                4828,
                4822,
                4818,
                4814,
                4808,
                4804,
                4800,
                4794,
                4790,
                4786,
                4780,
                4776,
                4772,
                4766,
                4762,
                4758,
                4752,
                4748,
                4744,
                4738,
                4734,
                4728,
                4724,
                4720,
                4714,
                4710,
                4706,
                4700,
                4696,
                4690,
                4686,
                4682,
                4676,
                4672,
                4666,
                4662,
                4658,
                4652,
                4648,
                4642,
                4638,
                4634,
                4628,
                4624,
                4618,
                4614,
                4610,
                4604,
                4600,
                4594,
                4590,
                4586,
                4580,
                4576,
                4570,
                4566,
                4560,
                4556,
                4552,
                4546,
                4542,
                4536,
                4532,
                4526,
                4522,
                4516,
                4512,
                4508,
                4502,
                4498,
                4492,
                4488,
                4482,
                4478,
                4472,
                4468,
                4462,
                4458,
                4454,
                4448,
                4444,
                4438,
                4434,
                4428,
                4424,
                4418,
                4414,
                4408,
                4404,
                4398,
                4394,
                4388,
                4384,
                4378,
                4374,
                4368,
                4364,
                4358,
                4354,
                4348,
                4344,
                4338,
                4334,
                4328,
                4324,
                4318,
                4314,
                4308,
                4304,
                4298,
                4294,
                4288,
                4284,
                4278,
                4274,
                4268,
                4262,
                4258,
                4252,
                4248,
                4242,
                4238,
                4232,
                4228,
                4222,
                4218,
                4212,
                4208,
                4202,
                4196,
                4192,
                4186,
                4182,
                4176,
                4172,
                4166,
                4162,
                4156,
                4150,
                4146,
                4140,
                4136,
                4130,
                4126,
                4120,
                4114,
                4110,
                4104,
                4100,
                4094,
                4088,
                4084,
                4078,
                4074,
                4068,
                4064,
                4058,
                4052,
                4048,
                4042,
                4038,
                4032,
                4026,
                4022,
                4016,
                4012,
                4006,
                4000,
                3996,
                3990,
                3984,
                3980,
                3974,
                3970,
                3964,
                3958,
                3954,
                3948,
                3942,
                3938,
                3932,
                3928,
                3922,
                3916,
                3912,
                3906,
                3900,
                3896,
                3890,
                3884,
                3880,
                3874,
                3870,
                3864,
                3858,
                3854,
                3848,
                3842,
                3838,
                3832,
                3826,
                3822,
                3816,
                3810,
                3806,
                3800,
                3794,
                3790,
                3784,
                3778,
                3774,
                3768,
                3762,
                3758,
                3752,
                3746,
                3742,
                3736,
                3730,
                3724,
                3720,
                3714,
                3708,
                3704,
                3698,
                3692,
                3688,
                3682,
                3676,
                3672,
                3666,
                3660,
                3654,
                3650,
                3644,
                3638,
                3634,
                3628,
                3622,
                3616,
                3612,
                3606,
                3600,
                3596,
                3590,
                3584,
                3578,
                3574,
                3568,
                3562,
                3556,
                3552,
                3546,
                3540,
                3536,
                3530,
                3524,
                3518,
                3514,
                3508,
                3502,
                3496,
                3492,
                3486,
                3480,
                3474,
                3470,
                3464,
                3458,
                3452,
                3448,
                3442,
                3436,
                3430,
                3424,
                3420,
                3414,
                3408,
                3402,
                3398,
                3392,
                3386,
                3380,
                3376,
                3370,
                3364,
                3358,
                3352,
                3348,
                3342,
                3336,
                3330,
                3326,
                3320,
                3314,
                3308,
                3302,
                3298,
                3292,
                3286,
                3280,
                3274,
                3270,
                3264,
                3258,
                3252,
                3246,
                3242,
                3236,
                3230,
                3224,
                3218,
                3214,
                3208,
                3202,
                3196,
                3190,
                3184,
                3180,
                3174,
                3168,
                3162,
                3156,
                3152,
                3146,
                3140,
                3134,
                3128,
                3122,
                3118,
                3112,
                3106,
                3100,
                3094,
                3088,
                3084,
                3078,
                3072,
                3066,
                3060,
                3054,
                3050,
                3044,
                3038,
                3032,
                3026,
                3020,
                3014,
                3010,
                3004,
                2998,
                2992,
                2986,
                2980,
                2974,
                2970,
                2964,
                2958,
                2952,
                2946,
                2940,
                2934,
                2930,
                2924,
                2918,
                2912,
                2906,
                2900,
                2894,
                2888,
                2884,
                2878,
                2872,
                2866,
                2860,
                2854,
                2848,
                2842,
                2836,
                2832,
                2826,
                2820,
                2814,
                2808,
                2802,
                2796,
                2790,
                2784,
                2780,
                2774,
                2768,
                2762,
                2756,
                2750,
                2744,
                2738,
                2732,
                2726,
                2720,
                2716,
                2710,
                2704,
                2698,
                2692,
                2686,
                2680,
                2674,
                2668,
                2662,
                2656,
                2650,
                2646,
                2640,
                2634,
                2628,
                2622,
                2616,
                2610,
                2604,
                2598,
                2592,
                2586,
                2580,
                2574,
                2570,
                2564,
                2558,
                2552,
                2546,
                2540,
                2534,
                2528,
                2522,
                2516,
                2510,
                2504,
                2498,
                2492,
                2486,
                2480,
                2474,
                2468,
                2464,
                2458,
                2452,
                2446,
                2440,
                2434,
                2428,
                2422,
                2416,
                2410,
                2404,
                2398,
                2392,
                2386,
                2380,
                2374,
                2368,
                2362,
                2356,
                2350,
                2344,
                2338,
                2332,
                2326,
                2320,
                2314,
                2308,
                2302,
                2298,
                2292,
                2286,
                2280,
                2274,
                2268,
                2262,
                2256,
                2250,
                2244,
                2238,
                2232,
                2226,
                2220,
                2214,
                2208,
                2202,
                2196,
                2190,
                2184,
                2178,
                2172,
                2166,
                2160,
                2154,
                2148,
                2142,
                2136,
                2130,
                2124,
                2118,
                2112,
                2106,
                2100,
                2094,
                2088,
                2082,
                2076,
                2070,
                2064,
                2058,
                2052,
                2046,
                2040,
                2034,
                2028,
                2022,
                2016,
                2010,
                2004,
                1998,
                1992,
                1986,
                1980,
                1974,
                1966,
                1960,
                1954,
                1948,
                1942,
                1936,
                1930,
                1924,
                1918,
                1912,
                1906,
                1900,
                1894,
                1888,
                1882,
                1876,
                1870,
                1864,
                1858,
                1852,
                1846,
                1840,
                1834,
                1828,
                1822,
                1816,
                1810,
                1804,
                1798,
                1792,
                1784,
                1778,
                1772,
                1766,
                1760,
                1754,
                1748,
                1742,
                1736,
                1730,
                1724,
                1718,
                1712,
                1706,
                1700,
                1694,
                1688,
                1682,
                1676,
                1670,
                1662,
                1656,
                1650,
                1644,
                1638,
                1632,
                1626,
                1620,
                1614,
                1608,
                1602,
                1596,
                1590,
                1584,
                1578,
                1572,
                1564,
                1558,
                1552,
                1546,
                1540,
                1534,
                1528,
                1522,
                1516,
                1510,
                1504,
                1498,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
                0,
            ]

    def second_axis_vals(self, first_val):
        first_idx = self._first_axis_vals.index(first_val[0])
        Ny = self.lon_spacing()[first_idx]
        if Ny != 0:
            second_spacing = 360 / Ny
            return [i * second_spacing for i in range(Ny)]
        else:
            raise ValueError("Requested latitude value is not within reduced latitude-longitude grid bounds")

    def map_second_axis(self, first_val, lower, upper):
        axis_lines = self.second_axis_vals(first_val)
        return_vals = [val for val in axis_lines if lower <= val <= upper]
        return return_vals

    def axes_idx_to_reduced_ll_idx(self, first_idx, second_idx):
        Ny_array = self.lon_spacing()
        idx = 0
        for i in range(self._resolution):
            if i != first_idx:
                idx += Ny_array[i]
            else:
                idx += second_idx
                return idx

    def find_second_idx(self, first_val, second_val):
        tol = 1e-10
        second_axis_vals = self.second_axis_vals(first_val)
        second_idx = bisect.bisect_left(second_axis_vals, second_val - tol)
        return second_idx

    def unmap(self, first_val, second_vals):
        tol = 1e-8
        first_value = [i for i in self._first_axis_vals if first_val[0] - tol <= i <= first_val[0] + tol][0]
        first_idx = self._first_axis_vals.index(first_value)
        return_idxs = []
        for second_val in second_vals:
            second_val = [i for i in self.second_axis_vals(first_val) if second_val - tol <= i <= second_val + tol][0]
            second_idx = self.second_axis_vals(first_val).index(second_val)
            reduced_ll_index = self.axes_idx_to_reduced_ll_idx(first_idx, second_idx)
            return_idxs.append(reduced_ll_index)
        return return_idxs


# md5 grid hash in form {resolution : hash}
_md5_hash = {3601: "386742a2dd1201b67f2d19ed421353ea"}
